package com.iot.modbus_rtcp.jobs;

import cn.hutool.core.map.MapUtil;
import com.iot.modbus_rtcp.controller.ModbusTCPController;
import com.iot.modbus_rtcp.dto.CommandTypeComparable;
import com.iot.modbus_rtcp.dto.ModbusCommandDto;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 18:33
 */
@Component
@RequiredArgsConstructor
public class AutoCollectJobs {

    private static final String SQL = "SELECT c.id AS commandId,c.ref_id AS deviceId,c.command, " +
            "c.message_length messageLength,d.gateway_sn AS identifier FROM commands c JOIN device d " +
            "ON c.ref_id = d.id WHERE c.type = 'COLLECTION' AND c.ref_type = 'DEVICE' ORDER BY c.id ";

    private final JdbcTemplate jdbcTemplate;
    private final ModbusTCPController controller;
    private final ThreadPoolExecutor collectThreadPool = new ThreadPoolExecutor(Math.min(10, Runtime.getRuntime().availableProcessors())
            , Math.min(100, Runtime.getRuntime().availableProcessors()), 60, TimeUnit.SECONDS,
            new ArrayBlockingQueue<>(5000), new ThreadFactory() {
        AtomicInteger index = new AtomicInteger(1);

        @Override
        public Thread newThread(Runnable runnable) {
            Thread thread = new Thread(runnable, "Modbus-auto-collect-thread-" + index.getAndIncrement());
            thread.setDaemon(true);
            thread.setPriority(Thread.MIN_PRIORITY);
            return thread;
        }
    }, new ThreadPoolExecutor.CallerRunsPolicy());

    @Scheduled(cron = "0/30 * * * * ? ")
    public void autoCollect() {
        int pageIndex = 0;
        int pageSize = 1000;
        long timestamp = System.currentTimeMillis();
        List<Map<String, Object>> resultList = null;
        while (Objects.isNull(resultList) || resultList.size() >= pageSize) {
            resultList = this.jdbcTemplate.queryForList(SQL + " LIMIT " + (pageIndex++) + "," + pageSize);
            if (ObjectUtils.isEmpty(resultList)) {
                continue;
            }
            List<ModbusCommandDto> collectCommondList = resultList.stream()
                    .filter(item -> Objects.nonNull(ModbusTCPController.nettyServer.getGroup()
                            .get((MapUtil.getStr(item, "identifier")))))
                    .map(item -> {
                        String identifier = MapUtil.getStr(item, "identifier");
                        return ModbusCommandDto.builder()
                                .command(MapUtil.getStr(item, "command"))
                                .identifier(identifier)
                                .length(MapUtil.getInt(item, "messageLength"))
                                .type(CommandTypeComparable.CommandType.COLLECTION)
                                .key(StringUtils.joinWith("/", identifier,
                                        MapUtil.getStr(item, "deviceId"),
                                        MapUtil.getStr(item, "commandId"),
                                        timestamp))
                                .build();
                    })
                    .collect(Collectors.toList());
            if (ObjectUtils.isEmpty(collectCommondList)) {
                continue;
            }
            this.collectThreadPool.execute(() -> this.controller.collect(collectCommondList));
        }
    }
}
